/*==============================================================================


       dlg_font - Font Selection Dialog Example

              Copyright  2001 Pankhurst Algorithmics - All Rights Reserved
              visit www.ebmDevMag.com for permissions/restrictions on use


==============================================================================*/
#include "gui.h"
#include "ebm_object.h"
/*==============================================================================


       Font Selection Dialog - consists of:


       1) a CStructuredList class for the list


       2) a CStructuredPopUp window wrapper


       3) the PopUp_FontSelect() function


       To use, call PopUp_FontSelect() - the return value is a valid FONT

       pointer to use, or zero if cancelled


==============================================================================*/

class CStructuredList : public CList
{


  private:

    CWindow *m_window;
    const FONTGROUP **m_array; // pointer to our structure's data
    const char *m_base; // FONTGROUP base - needed to find specific font

    static const char *Description[];

    U16 GetRowHeight(S32 row) // draw row in relation to font height
    {
      if (NULL==m_window)
        m_window=GetWindow();
      return m_array[row]->size+5;
    }
    void DrawRow(RECT *rect, S32 row);

  public:

    CStructuredList(int id,int dx,int dy,const FONTGROUP *base,const FONTGROUP **ptr,int numRows)
     : CList(id,dx,dy,numRows,0),
       m_window(0),
       m_array(ptr),
       m_base((const char *)base)
    { }

};
//------------------------------------------------------------------------------
// descriptions of each CTRL_ font flag - each value is an index into this list
//------------------------------------------------------------------------------
/*static*/const char * CStructuredList::Description[] =
  { "Unknown", "Normal", "Bold", "Italic", "SuperScript", "Subscript", "Bold Italic", "Small" };
//------------------------------------------------------------------------------
void CStructuredList::DrawRow(RECT *rect, S32 row)
{
  // we convert row information into a text string, as well as font description
  // and print in the appropriate font. Finally, we draw lines to help show
  // where font is in relation to baseline (super- and sub-scripts for example)
  char buff[99]="";
  sprintf( buff, "%d pixel %s", // descriptive string
                 m_array[row]->size,
                 Description[m_array[row]->style] );
  // get font - we've cast m_base to char to make calculation less messy
  const FONT *font=(const FONT *)&m_base[m_array[row]->font];
  m_window->DrawText( buff, rect->x, rect->y+3, font );
  m_window->DrawLine( rect->x, // bottom line - always needed
                      rect->y+rect->height,
                      rect->x+rect->width,
                      rect->y+rect->height,
                      COLOR_BLACK );
  if (row==GetTopRow()) // first entry in list gets an upper line as well as bottom one
    m_window->DrawLine( rect->x,
                        rect->y,
                        rect->x+rect->width,
                        rect->y,
                        COLOR_BLACK );
}
//------------------------------------------------------------------------------
//     CStructuredPopUp - wrapper for CStructuredList
//------------------------------------------------------------------------------
class CStructuredPopUp : public CWindow
{

  private:

    static int s_choice;

    S32 MsgHandler( MSG_TYPE type, CViewable *from, S32 data );

  public:

    CStructuredPopUp(int x,int y,int dx,int dy,
                     const FONTGROUP *base,
                     const FONTGROUP **ptr,int numRows,
                     const char *caption);

    static int Choice(void)
    {  return s_choice;  }

};
//------------------------------------------------------------------------------
/*static*/ int CStructuredPopUp::s_choice;
//------------------------------------------------------------------------------
CStructuredPopUp::CStructuredPopUp(int x,int y,int dx,int dy,
                                   const FONTGROUP *base,
                                   const FONTGROUP **array,int numRows,
                                   const char *caption)
 : CWindow(100,x,y,dx,dy,caption,false,false,false)
{
  AddChild( new CButton(101,dx-8,20,"Cancel"),                       2,dy-40);
  AddChild( new CStructuredList(102,dx-10,dy-48,base,array,numRows), 5,    5);
}
//------------------------------------------------------------------------------
S32 CStructuredPopUp::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  if ( MSG_ROW_SELECT==type )
  {
    s_choice=data;
    Close();
  }
  else if ( MSG_BUTTON_SELECT==type )
  {
    s_choice=-1;
    Close();
  }
  return CWindow::MsgHandler(type, from, data);
}
//------------------------------------------------------------------------------
//       PopUp_FontSelect function
//------------------------------------------------------------------------------
const FONT *PopUp_FontSelect(void)
{
  // get current font group (Default unless changed)
  const FONTGROUP *group=GUI_GetFontGroup();
  // find out # entries and set up an array for them
  unsigned int total=0, i=0, n=0;
  while (group[total].size) // count group size - last entry has zero size
    ++total;
  const FONTGROUP **array=new const FONTGROUP*[total];
  for (i=0;i<total;i++) // check and add valid entries only
  {
    if ( group[i].size<48 && group[i].style<=CTRL_SMALL && group[i].font )
      array[n++]=&group[i];
  }
  // call popup dialog
  GUI_EventLoop(new CStructuredPopUp(10,10,180,210,group,array,n,"Select a Font"));
  int choice=CStructuredPopUp::Choice();
  const FONT *font=0;
  if (choice>=0) // selection made?
    font=(const FONT *)(((char *)group)+array[choice]->font); // calc font
  delete[] array;
  return font;
}
//------------------------------------------------------------------------------

